@@ -4,25 +4,29 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
+#include <config-libkleo.h>
+
 #include "keyselectioncombo.h"
+
+#include "progressbar.h"
+
+#include <libkleo/defaultkeyfilter.h>
+#include <libkleo/dn.h>
+#include <libkleo/formatting.h>
+#include <libkleo/keycache.h>
+#include <libkleo/keylist.h>
+#include <libkleo/keylistmodel.h>
+#include <libkleo/keylistsortfilterproxymodel.h>
+
 #include <kleo_ui_debug.h>
 
-#include "kleo/dn.h"
-#include "models/keylist.h"
-#include "models/keylistmodel.h"
-#include "models/keylistsortfilterproxymodel.h"
-#include "models/keycache.h"
-#include "utils/formatting.h"
-#include "progressbar.h"
-#include "kleo/defaultkeyfilter.h"
-
-#include <gpgme++/key.h>
+#include <KLocalizedString>
 
 #include <QSortFilterProxyModel>
-#include <QVector>
 #include <QTimer>
+#include <QVector>
 
-#include <KLocalizedString>
+#include <gpgme++/key.h>
 
 using namespace Kleo;
 
@@ -75,6 +79,7 @@ private:
         QVariant data;
         QString toolTip;
     };
+
 public:
     ProxyModel(QObject *parent = nullptr)
         : QSortFilterProxyModel(parent)
@@ -89,7 +94,7 @@ public:
 
     bool lessThan(const QModelIndex &left, const QModelIndex &right) const override
     {
-        const auto leftKey =  sourceModel()->data(left, KeyList::KeyRole).value<GpgME::Key>();
+        const auto leftKey = sourceModel()->data(left, KeyList::KeyRole).value<GpgME::Key>();
         const auto rightKey = sourceModel()->data(right, KeyList::KeyRole).value<GpgME::Key>();
         if (leftKey.isNull()) {
             return false;
@@ -106,7 +111,7 @@ public:
         if (rUid.isNull()) {
             return true;
         }
-        int cmp = strcmp (lUid.id(), rUid.id());
+        int cmp = strcmp(lUid.id(), rUid.id());
         if (cmp) {
             return cmp < 0;
         }
@@ -114,16 +119,16 @@ public:
         if (lUid.validity() == rUid.validity()) {
             /* Both are the same check which one is newer. */
             time_t oldTime = 0;
-            for (const GpgME::Subkey &s: leftKey.subkeys()) {
+            for (const GpgME::Subkey &s : leftKey.subkeys()) {
                 if (s.isRevoked() || s.isInvalid() || s.isDisabled()) {
                     continue;
                 }
                 if (s.creationTime() > oldTime) {
-                    oldTime= s.creationTime();
+                    oldTime = s.creationTime();
                 }
             }
             time_t newTime = 0;
-            for (const GpgME::Subkey &s: rightKey.subkeys()) {
+            for (const GpgME::Subkey &s : rightKey.subkeys()) {
                 if (s.isRevoked() || s.isInvalid() || s.isDisabled()) {
                     continue;
                 }
@@ -144,14 +149,14 @@ public:
     void prependItem(const QIcon &icon, const QString &text, const QVariant &data, const QString &toolTip)
     {
         beginInsertRows(QModelIndex(), 0, 0);
-        mFrontItems.push_front(new CustomItem{ icon, text, data, toolTip });
+        mFrontItems.push_front(new CustomItem{icon, text, data, toolTip});
         endInsertRows();
     }
 
     void appendItem(const QIcon &icon, const QString &text, const QVariant &data, const QString &toolTip)
     {
         beginInsertRows(QModelIndex(), rowCount(), rowCount());
-        mBackItems.push_back(new CustomItem{ icon, text, data, toolTip });
+        mBackItems.push_back(new CustomItem{icon, text, data, toolTip});
         endInsertRows();
     }
 
@@ -181,6 +186,15 @@ public:
         return mFrontItems.count() + QSortFilterProxyModel::rowCount(parent) + mBackItems.count();
     }
 
+    int columnCount(const QModelIndex &parent = QModelIndex()) const override
+    {
+        Q_UNUSED(parent)
+        // pretend that there is only one column to workaround a bug in
+        // QAccessibleTable which provides the accessibility interface for the
+        // pop-up of the combo box
+        return 1;
+    }
+
     QModelIndex mapToSource(const QModelIndex &index) const override
     {
         if (!isCustomItem(index.row())) {
@@ -233,7 +247,7 @@ public:
 
         if (isCustomItem(index.row())) {
             Q_ASSERT(!mFrontItems.isEmpty() || !mBackItems.isEmpty());
-            auto ci = static_cast<CustomItem*>(index.internalPointer());
+            auto ci = static_cast<CustomItem *>(index.internalPointer());
             switch (role) {
             case Qt::DisplayRole:
                 return ci->text;
@@ -268,20 +282,26 @@ public:
                 name = dn[QStringLiteral("CN")];
                 email = dn[QStringLiteral("EMAIL")];
             }
-            return i18nc("Name <email> (validity, type, created: date)", "%1 (%2, %3 created: %4)",
-                         email.isEmpty() ? name : name.isEmpty() ? email : i18nc("Name <email>", "%1 <%2>", name, email),
-                         Kleo::Formatting::complianceStringShort(key),
-                         Kleo::KeyCache::instance()->pgpOnly() ? QString() :
-                            key.protocol() == GpgME::OpenPGP ? i18n("OpenPGP") + QLatin1Char(',') : i18n("S/MIME") + QLatin1Char(','),
-                         Kleo::Formatting::creationDateString(key));
+            const auto nameAndEmail = email.isEmpty() ? name : name.isEmpty() ? email : i18nc("Name <email>", "%1 <%2>", name, email);
+            if (Kleo::KeyCache::instance()->pgpOnly()) {
+                return i18nc("Name <email> (validity, created: date)",
+                             "%1 (%2, created: %3)",
+                             nameAndEmail,
+                             Kleo::Formatting::complianceStringShort(key),
+                             Kleo::Formatting::creationDateString(key));
+            } else {
+                return i18nc("Name <email> (validity, type, created: date)",
+                             "%1 (%2, %3, created: %4)",
+                             nameAndEmail,
+                             Kleo::Formatting::complianceStringShort(key),
+                             Formatting::displayName(key.protocol()),
+                             Kleo::Formatting::creationDateString(key));
+            }
+        }
+        case Qt::ToolTipRole: {
+            using namespace Kleo::Formatting;
+            return Kleo::Formatting::toolTip(key, Validity | Issuer | Subject | Fingerprint | ExpiryDates | UserIDs);
         }
-        case Qt::ToolTipRole:
-            return Kleo::Formatting::toolTip(key, Kleo::Formatting::Validity |
-                                                  Kleo::Formatting::Issuer |
-                                                  Kleo::Formatting::Subject |
-                                                  Kleo::Formatting::Fingerprint |
-                                                  Kleo::Formatting::ExpiryDates |
-                                                  Kleo::Formatting::UserIDs);
         case Qt::DecorationRole:
             return Kleo::Formatting::iconForUid(key.userID(0));
         default:
@@ -290,11 +310,10 @@ public:
     }
 
 private:
-    QVector<CustomItem*> mFrontItems;
-    QVector<CustomItem*> mBackItems;
+    QVector<CustomItem *> mFrontItems;
+    QVector<CustomItem *> mBackItems;
 };
 
-
 } // anonymous namespace
 
 namespace Kleo
@@ -336,7 +355,7 @@ public:
                 // WTF?
                 continue;
             }
-            for (const auto &uid: key.userIDs()) {
+            for (const auto &uid : key.userIDs()) {
                 if (QString::fromStdString(uid.addrSpec()) == mPerfectMatchMbox) {
                     q->setCurrentIndex(i);
                     return true;
@@ -348,20 +367,21 @@ public:
 
     /* Updates the current key with the default key if the key matches
      * the current key filter. */
-    void updateWithDefaultKey() {
+    void updateWithDefaultKey()
+    {
         GpgME::Protocol filterProto = GpgME::UnknownProtocol;
 
-        const auto filter = dynamic_cast<const DefaultKeyFilter*> (sortFilterProxy->keyFilter().get());
+        const auto filter = dynamic_cast<const DefaultKeyFilter *>(sortFilterProxy->keyFilter().get());
         if (filter && filter->isOpenPGP() == DefaultKeyFilter::Set) {
             filterProto = GpgME::OpenPGP;
         } else if (filter && filter->isOpenPGP() == DefaultKeyFilter::NotSet) {
             filterProto = GpgME::CMS;
         }
 
-        QString defaultKey = defaultKeys.value (filterProto);
+        QString defaultKey = defaultKeys.value(filterProto);
         if (defaultKey.isEmpty()) {
             // Fallback to unknown protocol
-            defaultKey = defaultKeys.value (GpgME::UnknownProtocol);
+            defaultKey = defaultKeys.value(GpgME::UnknownProtocol);
         }
         // make sure that the default key is not filtered out unless it has the wrong protocol
         if (filterProto == GpgME::UnknownProtocol) {
@@ -411,18 +431,19 @@ public:
     QVariant customItemBeforeModelChange;
 
 private:
-    KeySelectionCombo * const q;
+    KeySelectionCombo *const q;
 };
 
 }
 
 using namespace Kleo;
 
-KeySelectionCombo::KeySelectionCombo(QWidget* parent)
+KeySelectionCombo::KeySelectionCombo(QWidget *parent)
     : KeySelectionCombo(true, parent)
-{}
+{
+}
 
-KeySelectionCombo::KeySelectionCombo(bool secretOnly, QWidget* parent)
+KeySelectionCombo::KeySelectionCombo(bool secretOnly, QWidget *parent)
     : QComboBox(parent)
     , d(new KeySelectionComboPrivate(this))
 {
@@ -436,31 +457,36 @@ KeySelectionCombo::KeySelectionCombo(bool secretOnly, QWidget* parent)
     d->proxyModel->setSourceModel(d->sortFilterProxy);
 
     setModel(d->proxyModel);
-    connect(this, qOverload<int>(&QComboBox::currentIndexChanged),
-            this, [this](int row) {
-                if (row >= 0 && row < d->proxyModel->rowCount()) {
-                    if (d->proxyModel->isCustomItem(row)) {
-                        Q_EMIT customItemSelected(currentData(Qt::UserRole));
-                    } else {
-                        Q_EMIT currentKeyChanged(currentKey());
-                    }
-                }
-            });
+    connect(this, qOverload<int>(&QComboBox::currentIndexChanged), this, [this](int row) {
+        if (row >= 0 && row < d->proxyModel->rowCount()) {
+            if (d->proxyModel->isCustomItem(row)) {
+                Q_EMIT customItemSelected(currentData(Qt::UserRole));
+            } else {
+                Q_EMIT currentKeyChanged(currentKey());
+            }
+        }
+    });
 
     d->cache = Kleo::KeyCache::mutableInstance();
 
-    connect(model(), &QAbstractItemModel::rowsAboutToBeInserted,
-            this, [this] () { d->storeCurrentSelectionBeforeModelChange(); });
-    connect(model(), &QAbstractItemModel::rowsInserted,
-            this, [this] () { d->restoreCurrentSelectionAfterModelChange(); });
-    connect(model(), &QAbstractItemModel::rowsAboutToBeRemoved,
-            this, [this] () { d->storeCurrentSelectionBeforeModelChange(); });
-    connect(model(), &QAbstractItemModel::rowsRemoved,
-            this, [this] () { d->restoreCurrentSelectionAfterModelChange(); });
-    connect(model(), &QAbstractItemModel::modelAboutToBeReset,
-            this, [this] () { d->storeCurrentSelectionBeforeModelChange(); });
-    connect(model(), &QAbstractItemModel::modelReset,
-            this, [this] () { d->restoreCurrentSelectionAfterModelChange(); });
+    connect(model(), &QAbstractItemModel::rowsAboutToBeInserted, this, [this]() {
+        d->storeCurrentSelectionBeforeModelChange();
+    });
+    connect(model(), &QAbstractItemModel::rowsInserted, this, [this]() {
+        d->restoreCurrentSelectionAfterModelChange();
+    });
+    connect(model(), &QAbstractItemModel::rowsAboutToBeRemoved, this, [this]() {
+        d->storeCurrentSelectionBeforeModelChange();
+    });
+    connect(model(), &QAbstractItemModel::rowsRemoved, this, [this]() {
+        d->restoreCurrentSelectionAfterModelChange();
+    });
+    connect(model(), &QAbstractItemModel::modelAboutToBeReset, this, [this]() {
+        d->storeCurrentSelectionBeforeModelChange();
+    });
+    connect(model(), &QAbstractItemModel::modelReset, this, [this]() {
+        d->restoreCurrentSelectionAfterModelChange();
+    });
 
     QTimer::singleShot(0, this, &KeySelectionCombo::init);
 }
@@ -469,36 +495,35 @@ KeySelectionCombo::~KeySelectionCombo() = default;
 
 void KeySelectionCombo::init()
 {
-    connect(d->cache.get(), &Kleo::KeyCache::keyListingDone,
-            this, [this]() {
-                    // Set useKeyCache ensures that the cache is populated
-                    // so this can be a blocking call if the cache is not initialized 
-                    if (!d->initialKeyListingDone) {
-                        d->model->useKeyCache(true, d->secretOnly ? KeyList::SecretKeysOnly : KeyList::AllKeys);
-                        d->proxyModel->removeCustomItem(QStringLiteral("-libkleo-loading-keys"));
-                    }
+    connect(d->cache.get(), &Kleo::KeyCache::keyListingDone, this, [this]() {
+        // Set useKeyCache ensures that the cache is populated
+        // so this can be a blocking call if the cache is not initialized
+        if (!d->initialKeyListingDone) {
+            d->model->useKeyCache(true, d->secretOnly ? KeyList::SecretKeysOnly : KeyList::AllKeys);
+            d->proxyModel->removeCustomItem(QStringLiteral("-libkleo-loading-keys"));
+        }
 
-                    // We use the useWasEnabled state variable to decide if we should
-                    // change the enable / disable state based on the keylist done signal.
-                    // If we triggered the refresh useWasEnabled is true and we want to
-                    // enable / disable again after our refresh, as the refresh disabled it.
-                    //
-                    // But if a keyListingDone signal comes from just a generic refresh
-                    // triggered by someone else we don't want to change the enable / disable
-                    // state.
-                    if (d->useWasEnabled) {
-                        setEnabled(d->wasEnabled);
-                        d->useWasEnabled = false;
-                    }
-                    Q_EMIT keyListingFinished();
-            });
+        // We use the useWasEnabled state variable to decide if we should
+        // change the enable / disable state based on the keylist done signal.
+        // If we triggered the refresh useWasEnabled is true and we want to
+        // enable / disable again after our refresh, as the refresh disabled it.
+        //
+        // But if a keyListingDone signal comes from just a generic refresh
+        // triggered by someone else we don't want to change the enable / disable
+        // state.
+        if (d->useWasEnabled) {
+            setEnabled(d->wasEnabled);
+            d->useWasEnabled = false;
+        }
+        Q_EMIT keyListingFinished();
+    });
 
     connect(this, &KeySelectionCombo::keyListingFinished, this, [this]() {
-            if (!d->initialKeyListingDone) {
-                d->updateWithDefaultKey();
-                d->initialKeyListingDone = true;
-            }
-        });
+        if (!d->initialKeyListingDone) {
+            d->updateWithDefaultKey();
+            d->initialKeyListingDone = true;
+        }
+    });
 
     if (!d->cache->initialized()) {
         refreshKeys();
@@ -507,12 +532,11 @@ void KeySelectionCombo::init()
         Q_EMIT keyListingFinished();
     }
 
-    connect(this, qOverload<int>(&QComboBox::currentIndexChanged), this, [this] () {
-            setToolTip(currentData(Qt::ToolTipRole).toString());
-        });
+    connect(this, qOverload<int>(&QComboBox::currentIndexChanged), this, [this]() {
+        setToolTip(currentData(Qt::ToolTipRole).toString());
+    });
 }
 
-
 void KeySelectionCombo::setKeyFilter(const std::shared_ptr<const KeyFilter> &kf)
 {
     d->sortFilterProxy->setKeyFilter(kf);
@@ -527,14 +551,14 @@ std::shared_ptr<const KeyFilter> KeySelectionCombo::keyFilter() const
 
 void KeySelectionCombo::setIdFilter(const QString &id)
 {
-    d->sortFilterProxy->setFilterRegExp(id);
+    d->sortFilterProxy->setFilterRegularExpression(id);
     d->mPerfectMatchMbox = id;
     d->updateWithDefaultKey();
 }
 
 QString KeySelectionCombo::idFilter() const
 {
-    return d->sortFilterProxy->filterRegExp().pattern();
+    return d->sortFilterProxy->filterRegularExpression().pattern();
 }
 
 GpgME::Key Kleo::KeySelectionCombo::currentKey() const
@@ -556,8 +580,7 @@ void Kleo::KeySelectionCombo::setCurrentKey(const GpgME::Key &key)
 void Kleo::KeySelectionCombo::setCurrentKey(const QString &fingerprint)
 {
     const auto cur = currentKey();
-    if (!cur.isNull() && !fingerprint.isEmpty() &&
-        fingerprint == QLatin1String(cur.primaryFingerprint())) {
+    if (!cur.isNull() && !fingerprint.isEmpty() && fingerprint == QLatin1String(cur.primaryFingerprint())) {
         // already set; still emit a changed signal because the current key may
         // have become the item at the current index by changes in the underlying model
         Q_EMIT currentKeyChanged(cur);
